@@ -269,6 +269,8 @@ void QAndroidCameraSession::applyResolution(const QSize &captureSize, bool resta
 
     // -- adjust resolution
     QSize adjustedViewfinderResolution;
+    const QList<QSize> previewSizes = m_camera->getSupportedPreviewSizes();
+
     const bool validCaptureSize = captureSize.width() > 0 && captureSize.height() > 0;
     if (validCaptureSize
             && m_camera->getPreferredPreviewSizeForVideo().isEmpty()) {
@@ -280,8 +282,6 @@ void QAndroidCameraSession::applyResolution(const QSize &captureSize, bool resta
         if (validCaptureSize)
             captureAspectRatio = qreal(captureSize.width()) / qreal(captureSize.height());
 
-        const QList<QSize> previewSizes = m_camera->getSupportedPreviewSizes();
-
         if (validCaptureSize) {
             // search for viewfinder resolution with the same aspect ratio
             qreal minAspectDiff = 1;
@@ -326,24 +326,31 @@ void QAndroidCameraSession::applyResolution(const QSize &captureSize, bool resta
     // -- Set values on camera
 
     // fix the resolution of output based on the orientation
-    QSize outputResolution = adjustedViewfinderResolution;
+    QSize cameraOutputResolution = adjustedViewfinderResolution;
+    QSize videoOutputResolution = adjustedViewfinderResolution;
+    QSize currentVideoOutputResolution = m_videoOutput ? m_videoOutput->getVideoSize() : QSize(0, 0);
     const int rotation = currentCameraRotation();
-    if (rotation == 90 || rotation == 270)
-        outputResolution.transpose();
+    // only transpose if it's valid for the preview
+    if (rotation == 90 || rotation == 270) {
+        videoOutputResolution.transpose();
+        if (previewSizes.contains(cameraOutputResolution.transposed()))
+            cameraOutputResolution.transpose();
+    }
 
-    if (currentViewfinderResolution != outputResolution
+    if (currentViewfinderResolution != cameraOutputResolution
+        || (m_videoOutput && currentVideoOutputResolution != videoOutputResolution)
         || currentPreviewFormat != adjustedPreviewFormat || currentFpsRange.min != adjustedFps.min
         || currentFpsRange.max != adjustedFps.max) {
 
         if (m_videoOutput) {
-            m_videoOutput->setVideoSize(outputResolution);
+            m_videoOutput->setVideoSize(videoOutputResolution);
         }
 
         // if preview is started, we have to stop it first before changing its size
         if (m_previewStarted && restartPreview)
             m_camera->stopPreview();
 
-        m_camera->setPreviewSize(outputResolution);
+        m_camera->setPreviewSize(cameraOutputResolution);
         m_camera->setPreviewFormat(adjustedPreviewFormat);
         m_camera->setPreviewFpsRange(adjustedFps);
 
@@ -613,7 +620,6 @@ int QAndroidCameraSession::captureImage()
 
     m_currentImageCaptureId = newImageCaptureId;
 
-    applyImageSettings();
     applyResolution(m_actualImageSettings.resolution());
     m_camera->takePicture();
 
@@ -690,12 +696,16 @@ void QAndroidCameraSession::onNewPreviewFrame(const QVideoFrame &frame)
     m_videoFrameCallbackMutex.unlock();
 }
 
-void QAndroidCameraSession::onCameraPictureCaptured(const QVideoFrame &frame)
+void QAndroidCameraSession::onCameraPictureCaptured(const QByteArray &bytes,
+                                QVideoFrameFormat::PixelFormat format, QSize size,int bytesPerLine)
 {
-    // Loading and saving the captured image can be slow, do it in a separate thread
-    (void)QtConcurrent::run(&QAndroidCameraSession::processCapturedImage, this,
-                            m_currentImageCaptureId, frame, m_imageCaptureToBuffer,
-                            m_currentImageCaptureFileName);
+    if (m_imageCaptureToBuffer) {
+        processCapturedImageToBuffer(m_currentImageCaptureId, bytes, format, size, bytesPerLine);
+    } else {
+        // Loading and saving the captured image can be slow, do it in a separate thread
+        (void)QtConcurrent::run(&QAndroidCameraSession::processCapturedImage, this,
+                         m_currentImageCaptureId, bytes, m_currentImageCaptureFileName);
+    }
 
     // Preview needs to be restarted after taking a picture
     if (m_camera)
@@ -733,37 +743,38 @@ void QAndroidCameraSession::onCameraPreviewStopped()
     setReadyForCapture(false);
 }
 
-void QAndroidCameraSession::processCapturedImage(int id, const QVideoFrame &frame,
-                                                 bool captureToBuffer, const QString &fileName)
+void QAndroidCameraSession::processCapturedImage(int id, const QByteArray &bytes, const QString &fileName)
 {
-    if (captureToBuffer) {
-        emit imageAvailable(id, frame);
-        return;
-    }
-
     const QString actualFileName = QMediaStorageLocation::generateFileName(
             fileName, QStandardPaths::PicturesLocation, QLatin1String("jpg"));
-    QImageWriter writer(actualFileName);
 
-    if (!writer.canWrite()) {
+    QFile writer(actualFileName);
+    if (!writer.open(QIODeviceBase::WriteOnly)) {
         const QString errorMessage = tr("File is not available: %1").arg(writer.errorString());
         emit imageCaptureError(id, QImageCapture::Error::ResourceError, errorMessage);
         return;
     }
 
-    const bool written = writer.write(frame.toImage());
-    if (!written) {
+    if (writer.write(bytes) < 0) {
         const QString errorMessage = tr("Could not save to file: %1").arg(writer.errorString());
         emit imageCaptureError(id, QImageCapture::Error::ResourceError, errorMessage);
         return;
     }
 
+    writer.close();
     if (fileName.isEmpty() || QFileInfo(fileName).isRelative())
         AndroidMultimediaUtils::registerMediaFile(actualFileName);
 
     emit imageSaved(id, actualFileName);
 }
 
+void QAndroidCameraSession::processCapturedImageToBuffer(int id, const QByteArray &bytes,
+                              QVideoFrameFormat::PixelFormat format, QSize size, int bytesPerLine)
+{
+    QVideoFrame frame(new QMemoryVideoBuffer(bytes, bytesPerLine), QVideoFrameFormat(size, format));
+    emit imageAvailable(id, frame);
+}
+
 void QAndroidCameraSession::onVideoOutputReady(bool ready)
 {
     if (ready && m_active)
